@@ -2,16 +2,22 @@
 // Licensed under the MIT license.
 // Author: Paul Koch <code@koch.ninja>
 
-#ifndef TRANSPARENT_ML_CORE_INTERNAL_H
-#define TRANSPARENT_ML_CORE_INTERNAL_H
+#ifndef EBM_INTERNAL_H
+#define EBM_INTERNAL_H
 
 #include <inttypes.h>
 #include <stddef.h> // size_t, ptrdiff_t
 #include <limits> // numeric_limits
+#include <type_traits> // is_integral
 
-#define INVALID_POINTER (reinterpret_cast<void *>(~size_t { 0 }))
 #define UNUSED(x) (void)(x)
+// UBSAN really doesn't like it when we access data past the end of a class eg( p->m_a[2], when m_a is declared as an array of 1)
+// We do this however in a number of places to co-locate memory for performance reasons.  We do allocate sufficient memory for doing this, and we also statically check that our classes are standard layout structures (even if declared as classes), so accessing that memory is legal.
+// this MACRO turns an array reference into a pointer to the same type of object, which resolves any UBSAN warnings
+
+// TODO : the const and non-const macros can probably be unified
 #define ARRAY_TO_POINTER(x) (reinterpret_cast<typename std::remove_all_extents<decltype(x)>::type *>(reinterpret_cast<void *>(x)))
+#define ARRAY_TO_POINTER_CONST(x) (reinterpret_cast<const typename std::remove_all_extents<decltype(x)>::type *>(reinterpret_cast<const void *>(x)))
 
 // here's how to detect the compiler type for a variety of compilers -> https://sourceforge.net/p/predef/wiki/Compilers/
 // disabling warnings with _Pragma detailed info here https://stackoverflow.com/questions/3378560/how-to-disable-gcc-warnings-for-a-few-lines-of-code
@@ -36,7 +42,7 @@
 
 #elif defined(__SUNPRO_CC) // compiler type (Oracle Developer Studio)
 
-// The Oracle compiler doesn't seem to have a way to push/pop error messages, but they do have the concept of the "default" which acts as a pop for the specific warning that we turned on/off
+// The Oracle Developer Studio compiler doesn't seem to have a way to push/pop warning/error messages, but they do have the concept of the "default" which acts as a pop for the specific warning that we turn on/off
 // Since we can only default on previously changed warnings, we need to have matching warnings off/default sets, so use WARNING_DEFAULT_* 
 // example: WARNING_DISABLE_SOMETHING   _Pragma("error_messages(off,something1,something2)")
 // example: WARNING_DEFAULT_SOMETHING   _Pragma("error_messages(default,something1,something2)")
@@ -103,14 +109,15 @@ WARNING_DISABLE_SIGNED_UNSIGNED_MISMATCH
 
 template<typename TTo, typename TFrom>
 constexpr EBM_INLINE bool IsNumberConvertable(const TFrom number) {
-   // TODO: this function won't work for some floats and doubles.  For example, if you had just a bit more than the maximum integer in a double, it might round down properly, but this function will say that it won't.
-
    // the general rules of conversion are as follows:
    // calling std::numeric_limits<?>::max() returns an item of that type
    // casting and comparing will never give us undefined behavior.  It can give us implementation defined behavior or unspecified behavior, which is legal.  Undefined behavior results from overflowing negative integers, but we don't add or subtract.
    // C/C++ uses value preserving instead of sign preserving.  Generally, if you have two integer numbers that you're comparing then if one type can be converted into the other with no loss in range then that the smaller range integer is converted into the bigger range integer
    // if one type can't cover the entire range of the other, then items are converted to UNSIGNED values.  This is probably the most dangerous thing for us to deal with
 
+   static_assert(std::is_integral<TTo>::value, "TTo must be integral");
+   static_assert(std::is_integral<TFrom>::value, "TFrom must be integral");
+
    static_assert(std::numeric_limits<TTo>::is_specialized, "TTo must be specialized");
    static_assert(std::numeric_limits<TFrom>::is_specialized, "TFrom must be specialized");
 
@@ -161,41 +168,35 @@ enum class FeatureTypeCore { OrdinalCore = 0, NominalCore = 1};
 // 16  => 5.12%
 // 8   => 5.34%
 // 4   => 8.31%
-// TODO: increase this up to something like 16.  I have decreased it to 2 in order to make compiling more efficient, and so that I regularily test the runtime looped version of our code
-// TODO: BUT DON'T CHANGE THIS VALUE FROM 2 UNTIL WE HAVE RESOLVED THE ISSUE OF Removing compilerLearningTypeOrCountTargetClasses from the main template of SegmentedRegion (using a template on just the Add function would be ok)
-constexpr ptrdiff_t k_cCompilerOptimizedTargetClassesMax = 3;
+// TODO: increase this up to something like 16.  I have decreased it to 8 in order to make compiling more efficient, and so that I regularily test the runtime looped version of our code
+constexpr ptrdiff_t k_cCompilerOptimizedTargetClassesMax = 8;
 static_assert(2 <= k_cCompilerOptimizedTargetClassesMax, "we special case binary classification to have only 1 output.  If we remove the compile time optimization for the binary class situation then we would output model files with two values instead of our special case 1");
 
-// TODO : eliminate this typedef.. we bitpack our memory now, so we'll always want to use the biggest chunk of memory possible, which will be size_t
 typedef size_t StorageDataTypeCore;
-
-// TODO : add a MinusOneSizet const (size_t)(-1) -> turn most ptrdiff_t into size_t and use this constant where we just need a single negative number
-// TODO : eliminate this typedef.. we bitpack our memory now, so we'll always want to use the biggest chunk of memory possible, which will be size_t
-// we get a signed/unsigned mismatch if we use size_t in SegmentedRegion because we use whole numbers there
-typedef ptrdiff_t ActiveDataType;
+typedef size_t ActiveDataType;
 
 constexpr ptrdiff_t k_Regression = -1;
 constexpr ptrdiff_t k_DynamicClassification = 0;
-constexpr EBM_INLINE bool IsRegression(const ptrdiff_t compilerLearningTypeOrCountTargetClasses) {
-   return k_Regression == compilerLearningTypeOrCountTargetClasses;
+constexpr EBM_INLINE bool IsRegression(const ptrdiff_t learningTypeOrCountTargetClasses) {
+   return k_Regression == learningTypeOrCountTargetClasses;
 }
-constexpr EBM_INLINE bool IsClassification(const ptrdiff_t compilerLearningTypeOrCountTargetClasses) {
-   return 0 <= compilerLearningTypeOrCountTargetClasses;
+constexpr EBM_INLINE bool IsClassification(const ptrdiff_t learningTypeOrCountTargetClasses) {
+   return 0 <= learningTypeOrCountTargetClasses;
 }
-constexpr EBM_INLINE bool IsBinaryClassification(const ptrdiff_t compilerLearningTypeOrCountTargetClasses) {
+constexpr EBM_INLINE bool IsBinaryClassification(const ptrdiff_t learningTypeOrCountTargetClasses) {
 #ifdef EXPAND_BINARY_LOGITS
-   return false;
+   return UNUSED(learningTypeOrCountTargetClasses), false;
 #else // EXPAND_BINARY_LOGITS
-   return 2 == compilerLearningTypeOrCountTargetClasses;
+   return 2 == learningTypeOrCountTargetClasses;
 #endif // EXPAND_BINARY_LOGITS
 }
 
-constexpr EBM_INLINE size_t GetVectorLengthFlatCore(const ptrdiff_t runtimeLearningTypeOrCountTargetClasses) {
-   // this will work for anything except if compilerLearningTypeOrCountTargetClasses is set to DYNAMIC_CLASSIFICATION which means we should have passed in the dynamic value since DYNAMIC_CLASSIFICATION is a constant that doesn't tell us anything about the real value
+constexpr EBM_INLINE size_t GetVectorLengthFlatCore(const ptrdiff_t learningTypeOrCountTargetClasses) {
+   // this will work for anything except if learningTypeOrCountTargetClasses is set to DYNAMIC_CLASSIFICATION which means we should have passed in the dynamic value since DYNAMIC_CLASSIFICATION is a constant that doesn't tell us anything about the real value
 #ifdef EXPAND_BINARY_LOGITS
-   return runtimeLearningTypeOrCountTargetClasses <= ptrdiff_t { 1 } ? size_t { 1 } : static_cast<size_t>(runtimeLearningTypeOrCountTargetClasses);
+   return learningTypeOrCountTargetClasses <= ptrdiff_t { 1 } ? size_t { 1 } : static_cast<size_t>(learningTypeOrCountTargetClasses);
 #else // EXPAND_BINARY_LOGITS
-   return runtimeLearningTypeOrCountTargetClasses <= ptrdiff_t { 2 } ? size_t { 1 } : static_cast<size_t>(runtimeLearningTypeOrCountTargetClasses);
+   return learningTypeOrCountTargetClasses <= ptrdiff_t { 2 } ? size_t { 1 } : static_cast<size_t>(learningTypeOrCountTargetClasses);
 #endif // EXPAND_BINARY_LOGITS
 }
 
@@ -225,13 +226,11 @@ constexpr size_t CountBitsRequiredPositiveMax() {
 
 constexpr size_t k_cBitsForSizeTCore = CountBitsRequiredPositiveMax<size_t>();
 // it's impossible for us to have more than k_cDimensionsMax dimensions.  Even if we had the minimum number of bin per variable (two), then we would have 2^N memory spaces at our binning step, and that would exceed our memory size if it's greater than the number of bits allowed in a size_t, so on a 64 bit machine, 64 dimensions is a hard maximum.  We can subtract one bit safely, since we know that the rest of our program takes some memory, denying the full 64 bits of memory available.  This extra bit is very helpful since we can then set the 64th bit without overflowing it inside loops and other places
-// TODO : we should check at startup if there are equal to or less than these number of dimensions, otherwise return an error.  I don't think we want to wait until we try allocating the memory to discover that we can't do it
 // TODO : we can restrict the dimensionatlity even more because HistogramBuckets aren't 1 byte, so we can see how many would fit into memory.  This isn't a big deal, but it could be nice if we generate static code to handle every possible valid dimension value
 constexpr size_t k_cDimensionsMax = k_cBitsForSizeTCore - 1;
 static_assert(k_cDimensionsMax < k_cBitsForSizeTCore, "reserve the highest bit for bit manipulation space");
 
 constexpr size_t k_cBitsForStorageType = CountBitsRequiredPositiveMax<StorageDataTypeCore>();
-constexpr size_t k_cCountItemsBitPackedMax = k_cBitsForStorageType; // if each item is a bit, then the number of items will equal the number of bits
 
 constexpr EBM_INLINE size_t GetCountItemsBitPacked(const size_t cBits) {
    return k_cBitsForStorageType / cBits;
@@ -247,8 +246,6 @@ constexpr EBM_INLINE size_t GetNextCountItemsBitPacked(const size_t cItemsBitPac
 
 WARNING_PUSH
 WARNING_DISABLE_POTENTIAL_DIVIDE_BY_ZERO
-// TODO : also check for places where to convert a size_t into a ptrdiff_t and check for overflow there throughout our code
-// TODO : there are many places that could be overflowing multiplication.  We need to look for places where this might happen
 constexpr EBM_INLINE bool IsMultiplyError(const size_t num1, const size_t num2) {
    // algebraically, we want to know if this is true: std::numeric_limits<size_t>::max() + 1 <= num1 * num2
    // which can be turned into: (std::numeric_limits<size_t>::max() + 1 - num1) / num1 + 1 <= num2
@@ -266,8 +263,8 @@ constexpr EBM_INLINE bool IsAddError(const size_t num1, const size_t num2) {
    return num1 + num2 < num1;
 }
 
-// TODO : keep this constant, but make it global and compile out the costs... we want to document that it's possible and how, but we have tested it and found it's worse
+// TODO eventually, eliminate these variables, and make eliminating logits a part of our regular framework
 static constexpr ptrdiff_t k_iZeroResidual = -1;
 static constexpr ptrdiff_t k_iZeroClassificationLogitAtInitialize = -1;
 
-#endif // TRANSPARENT_ML_CORE_INTERNAL_H
+#endif // EBM_INTERNAL_H
